@@ -11,7 +11,8 @@ InvalidQueryException = _querybuilder.InvalidQueryException
 
 def AND(*conditions):
     """
-    Similar to the :py:func:`.XOR` function except the boolean operator is `AND`
+    Similar to the :py:func:`.XOR` function except the boolean operator is
+    `AND`
     """
     assert conditions
     return _querybuilder.logical_and(conditions)
@@ -108,6 +109,13 @@ def UNIX_TIMESTAMP():
     return _querybuilder.unix_timestamp()
 
 
+def IF(clause, true_expr, false_expr):
+    """
+    The same as :py:func:`~.MAX` but instead generating a `MIN` function.
+    """
+    return _querybuilder.if_(clause, true_expr, false_expr)
+
+
 def create_function_builder(sql_func):
     """
     Create a custom SQL function builder. This allows you to extend the current
